

---------------------------------------------------------------------------------------------------------------------------------------------
-- Stored procedure TimeLimitsRenew
---------------------------------------------------------------------------------------------------------------------------------------------
-- Procedure which updates an existing TimeLimitsOfTypes-table
---------------------------------------------------------------------------------------------------------------------------------------------

IF EXISTS (SELECT * FROM dbo.sysobjects WHERE id = object_id(N'[dbo].[TimeLimitsRenew]') AND OBJECTPROPERTY(id, N'IsProcedure') = 1)
	DROP PROCEDURE [dbo].[TimeLimitsRenew]
GO
---------------------------------------------------------------------------------------------------------------------------------------------

CREATE PROCEDURE dbo.TimeLimitsRenew 
AS
SET NOCOUNT ON;

---------------------------------------------------------------------------------------------------------------------------------------------
-- Declaration and initialisation of variables
---------------------------------------------------------------------------------------------------------------------------------------------
DECLARE @trans int, 
		@months int, 
		@main int, 
		@prot int, 
		@lim int,
		@count int,
		@date dateTime,
		@minTime dateTime

SELECT	@trans = (SELECT UnitsToKeep FROM DeleteParameters),
		@months = (SELECT TimeLimitProtocol FROM DeleteParameters),
		@main = (SELECT CaseMain FROM DeleteParameters),
		@prot = (SELECT CaseProtocol FROM DeleteParameters),
		@lim = (SELECT TimeLimitMain FROM DeleteParameters),
		@minTime = (SELECT Min(Time) FROM MainData)

-- In case the database contains no data, Min(Time) equals null. 
-- In this case, minTime is set to the minimum of the two time limits (TimeLimitProtocol and TimeLimitMain). 
IF (@minTime IS NULL) BEGIN
	SELECT	@minTime = MIN(minTime) FROM (SELECT DATEADD("Month",-(@months),GETDATE()) as minTime UNION SELECT DATEADD("Month",-(@lim),GETDATE())) as minTime
END
---------------------------------------------------------------------------------------------------------------------------------------------

---------------------------------------------------------------------------------------------------------------------------------------------
-- Replicate TimeLimitsOfTypes to store the original PreviousTimeLimit values before calculating the new TimeLimitOfTypes 		
---------------------------------------------------------------------------------------------------------------------------------------------
SELECT * INTO TimeLimitsOfTypes2 FROM TimeLimitsOfTypes 
-- Remove all rows from TimeLimitsOfTypes table
TRUNCATE TABLE TimeLimitsOfTypes
---------------------------------------------------------------------------------------------------------------------------------------------

---------------------------------------------------------------------------------------------------------------------------------------------
-- Check for the different deletion cases and calculation of the ActualTimeLimit and PreviousTimeLimit
---------------------------------------------------------------------------------------------------------------------------------------------
IF (@main = 1) BEGIN
	INSERT INTO [TimeLimitsOfTypes] (Id, Name, ActualTimeLimit, PreviousTimeLimit)
	SELECT 0, '-', (SELECT DATEADD("MONTH", -(@lim), GETDATE())), @minTime

	UPDATE TimeLimitsOfTypes
	SET UnitsToDelete = (SELECT COUNT (*) FROM MainData WHERE MainData.Time < TimeLimitsOfTypes.ActualTimeLimit)

	UPDATE TimeLimitsOfTypes
	SET UnitsRetained = (SELECT COUNT (*) FROM MainData WHERE MainData.Time >= TimeLimitsOfTypes.ActualTimeLimit) 
END

-- Case 1: Only the @trans number of units are kept which were measured within the last @months months.  
IF (@prot = 1) BEGIN
	INSERT INTO [TimeLimitsOfTypes] (Id, Name, ActualTimeLimit, PreviousTimeLimit)
	SELECT TypeId.Id, TypeId.Name, 
		(select MAX(mt) from
		(
			SELECT MIN(time) as mt FROM 
			(SELECT TOP (@trans) time FROM MainData
			WHERE type = TypeId.Id		 
			ORDER BY [time] DESC) topN 
			union 
			select  DATEADD("Month", -(@months), GETDATE()) as mt
		) topN) as timelimit ,
	(select MIN(time) FROM MainData WHERE type = TypeId.Id) as mintime
	FROM TypeId 
	WHERE TypeId.Id <> 0
	ORDER BY TypeId.Id	
END

-- Case 2: In addition to keeping all units measured within the last @months months, this case guarantees that at least @trans number of units are kept. 
IF (@prot = 2) BEGIN
	INSERT INTO [TimeLimitsOfTypes] (Id, Name, ActualTimeLimit, PreviousTimeLimit)
	SELECT TypeId.Id, TypeId.Name, 
		(select MIN(mt) from
		(
			SELECT MIN(time) as mt FROM 
			(SELECT TOP (@trans) time FROM MainData
			WHERE type = TypeId.Id		 
			ORDER BY [time] DESC) topN 
			union 
			select  DATEADD("Month", -(@months), GETDATE()) as mt
		) topN) as timelimit ,
	(select MIN(time) FROM MainData WHERE type = TypeId.Id) as mintime
	FROM TypeId 
	WHERE TypeId.Id <> 0
	ORDER BY TypeId.Id	
END

-- Case 3: Only the last @trans number of units are kept.
IF (@prot = 3) BEGIN
	INSERT INTO [TimeLimitsOfTypes] (Id, Name, ActualTimeLimit, PreviousTimeLimit)
		SELECT TypeId.Id, TypeId.Name, 
		(select MIN(mt) from
		(
			SELECT MIN(time) as mt FROM 
			(SELECT TOP (@trans) time FROM MainData 
			WHERE type = TypeId.Id		 
			ORDER BY [time] DESC) topN 
		) topN) as timelimit ,
		(select MIN(time) FROM MainData WHERE type = TypeId.Id) as mintime
	FROM TypeId
	WHERE TypeId.Id <> 0
	ORDER BY TypeId.Id		
END

-- Case 4: All units measured within the last @months number of months are kept.
IF (@prot = 4) BEGIN
	INSERT INTO [TimeLimitsOfTypes] (Id, Name, ActualTimeLimit, PreviousTimeLimit)
		SELECT TypeId.Id, TypeId.Name, 
		(SELECT DATEADD("MONTH", -(@months), GETDATE())) ,
		(SELECT MIN(time) FROM MainData WHERE type = TypeId.Id) as mintime
	FROM TypeId
	WHERE TypeId.Id <> 0
	ORDER BY TypeId.Id		
END
---------------------------------------------------------------------------------------------------------------------------------------------

---------------------------------------------------------------------------------------------------------------------------------------------
-- Type-specific deletion: Fill in original PreviousTimeLimit values   
---------------------------------------------------------------------------------------------------------------------------------------------
IF (@prot <> 0) BEGIN
	UPDATE [TimeLimitsOfTypes]
	SET TimeLimitsOfTypes.PreviousTimeLimit = TimeLimitsOfTypes2.PreviousTimeLimit
	FROM TimeLimitsOfTypes INNER JOIN TimeLimitsOfTypes2 On TimeLimitsOfTypes.Id = TimeLimitsOfTypes2.Id
	WHERE TimeLimitsOfTypes.Id <> 0		
END
---------------------------------------------------------------------------------------------------------------------------------------------

---------------------------------------------------------------------------------------------------------------------------------------------
-- MainData deletion: In case data is deleted from both ProtocolData and MainData  
---------------------------------------------------------------------------------------------------------------------------------------------
-- If the PreviousTimeLimit (PTL) of the type-specific deletion is less than the ActualTimeLimit of the MainData deletion, 
-- then set the PreviousTimeLimit to the ActualTimeLimit
IF (@main = 1) BEGIN
	SELECT @date = (SELECT ActualTimeLimit FROM TimeLimitsOfTypes WHERE Id = 0)
	UPDATE [TimeLimitsOfTypes]
	SET PreviousTimeLimit = @date
	WHERE Id <> 0 AND PreviousTimeLimit <= @date
END
---------------------------------------------------------------------------------------------------------------------------------------------

---------------------------------------------------------------------------------------------------------------------------------------------
-- Type-specific deletion: Validity queries and calculation of the units to delete and to keep
---------------------------------------------------------------------------------------------------------------------------------------------
IF (@prot <> 0) BEGIN

-- If one of the queries above returns a null value for the ActualTimeLimit (e.g. in case there are no measurements 
-- for a type which has already been initialised), then set the ActualTimeLimit to @minTime
	UPDATE [TimeLimitsOfTypes]
		Set PreviousTimeLimit = @minTime
		WHERE PreviousTimeLimit IS NULL
	UPDATE [TimeLimitsOfTypes]
		Set ActualTimeLimit = @minTime
		WHERE ActualTimeLimit IS NULL

-- In case the ActualTimeLimit is less than the PreviousTimeLimit then set ActualTimeLimit to the PreviousTimeLimit
-- Otherwise the following calculation will result in an erroneous number of kept units
	UPDATE [TimeLimitsOfTypes]
		Set ActualTimeLimit = PreviousTimeLimit
		WHERE ActualTimeLimit < PreviousTimeLimit

-- Calculation of the number of units to be deleted and to be kept 
	UPDATE [TimeLimitsOfTypes]
	SET UnitsToDelete = (SELECT COUNT (*) FROM MainData WHERE MainData.Type = TimeLimitsOfTypes.Id AND
		MainData.Time < TimeLimitsOfTypes.ActualTimeLimit AND MainData.Time >= TimeLimitsOfTypes.PreviousTimeLimit)
	WHERE Id <> 0
	UPDATE [TimeLimitsOfTypes]
	SET UnitsRetained = (SELECT COUNT (*) FROM MainData WHERE MainData.Type = TimeLimitsOfTypes.Id AND 
		MainData.Time >= TimeLimitsOfTypes.ActualTimeLimit)
	WHERE Id <> 0
END
---------------------------------------------------------------------------------------------------------------------------------------------

---------------------------------------------------------------------------------------------------------------------------------------------
-- Set delete flag of TimeLimitsOfTypes
---------------------------------------------------------------------------------------------------------------------------------------------
-- In case the UnitsToDelete is zero (for a given type), then set column 'Deleted' to 1 
UPDATE [TimeLimitsOfTypes]
SET Deleted = 1
WHERE TimeLimitsOfTypes.UnitsToDelete = 0

-- In case the UnitsToDelete is NOT zero (for a given type), then set column 'Deleted' to 1 
UPDATE [TimeLimitsOfTypes]
SET Deleted = 0
WHERE TimeLimitsOfTypes.UnitsToDelete <> 0
---------------------------------------------------------------------------------------------------------------------------------------------

---------------------------------------------------------------------------------------------------------------------------------------------
-- Set renew flag of TimeLimitsOfTypes 
---------------------------------------------------------------------------------------------------------------------------------------------
-- In case there are still units left to delete, set 'Renew' in table 'DeleteParameters' to 0. 
SELECT @count = (SELECT COUNT (*) FROM TimeLimitsOfTypes 
				WHERE TimeLimitsOfTypes.UnitsToDelete <> 0)
If (@count <> 0) Begin
	Update [DeleteParameters]
	Set [RenewLimits] = 0
End
---------------------------------------------------------------------------------------------------------------------------------------------

-- Drop duplicate table TimeLimitsOfTypes2 
DROP TABLE TimeLimitsOfTypes2
GO
---------------------------------------------------------------------------------------------------------------------------------------------

PRINT 'Stored procedure added'
GO
