
USE MQ200B_ProdDb
GO

----------------------------------------------------------------------------------------------------------------------------
-- Dropping the old non-clustered indexes from ProtocolData to free up internal datafile space
----------------------------------------------------------------------------------------------------------------------------

-- Drop non-clustered index Idx_ProtocolData from ProtocolData
IF EXISTS (SELECT * FROM sysindexes WHERE name = N'Idx_ProtocolData')
DROP INDEX [Idx_ProtocolData] ON [dbo].[ProtocolData]
GO

-- Drop unique non-clustered index UQ_ProtocolData from ProtocolData
IF EXISTS (SELECT * FROM sysindexes WHERE name = N'UQ_ProtocolData')
ALTER TABLE [dbo].[ProtocolData] DROP CONSTRAINT [UQ_ProtocolData]
GO
----------------------------------------------------------------------------------------------------------------------------

----------------------------------------------------------------------------------------------------------------------------
-- Setting the auto shrink off, as this fragments the database indexes
----------------------------------------------------------------------------------------------------------------------------
DECLARE @dbName nvarchar(256)
SELECT @dbName = (SELECT DB_NAME() AS [Current Database])

EXECUTE( 'ALTER DATABASE ' + @dbName + ' SET AUTO_SHRINK OFF' )
GO
----------------------------------------------------------------------------------------------------------------------------

----------------------------------------------------------------------------------------------------------------------------
-- Adding a second filegroup to the database
----------------------------------------------------------------------------------------------------------------------------
PRINT 'Adding second filegroup to the database...'
GO

IF NOT EXISTS (SELECT name FROM sys.filegroups WHERE name = 'Protocol')
BEGIN
	RAISERROR('Adding secondary filegroup...',0,1) WITH NOWAIT
	DECLARE @dbName nvarchar(256)
	SELECT @dbName = (SELECT DB_NAME() AS [Current Database])

	EXECUTE( 'ALTER DATABASE ' + @dbName + ' ADD FILEGROUP Protocol' )
	RAISERROR('Secondary filegroup was added.',0,1) WITH NOWAIT
END
GO
----------------------------------------------------------------------------------------------------------------------------

DECLARE @start datetime,
		@end datetime 

SELECT @start = (SELECT MIN([Time]) FROM MainData WHERE YEAR([Time]) = 2017)
SELECT @end = (SELECT MAX([Time]) FROM MainData WHERE YEAR([Time]) = 2017)


-- Check if table secondary datafile already exists
IF NOT EXISTS (SELECT data_space_id FROM sys.database_files WHERE data_space_id = 2)
BEGIN
	RAISERROR('Adding secondary datafile...',0,1) WITH NOWAIT
	----------------------------------------------------------------------------------------------------------------------------
	-- Calculation of the size of the new secondary database file (to which the UnitsRetained from ProtocolData are to be copied)
	-- using a temporary table and the database stored procedure sp_spaceused
	----------------------------------------------------------------------------------------------------------------------------
	CREATE TABLE #RowCountsAndSizes (TableName NVARCHAR(128),rows CHAR(11),     
		   reserved VARCHAR(18),data VARCHAR(18),index_size VARCHAR(18),
		   unused VARCHAR(18))

	EXECUTE ('INSERT INTO #RowCountsAndSizes EXEC sp_spaceused ProtocolData')

	DECLARE @dbPath nvarchar(256), 
		@dbName nvarchar(256),  
		@size bigint,
		@nKeep bigint,
		@nMain int, 
		@sizeChar nvarchar(256), 
		@SQLString nvarchar(512)

	SELECT @dbName = (SELECT DB_NAME() AS [Current Database])
	-- Number of units to be retained and copied to the new secondary database file
	SELECT @nKeep = (SELECT Count(PkIdent) FROM MainData WHERE [Time] >= @start AND [Time] <= @end)
	-- Number of total units in the database
	SELECT @nMain = (SELECT Count(PkIdent) FROM MainData)
	-- Approximate space requirement of the new secondary database file
	SELECT @size = (SELECT CONVERT(bigint,left(data,len(data)-3)) FROM #RowCountsAndSizes) * @nKeep / @nMain
	IF @size < 1024 BEGIN SELECT @size = 1024 END

	-- Drop temporary table
	DROP TABLE #RowCountsAndSizes 

	-- Extraction of the path for the secondary database file
	SELECT @dbPath = LEFT((SELECT physical_name FROM sys.database_files WHERE type_desc = 'ROWS'),
	LEN((SELECT physical_name FROM sys.database_files WHERE type_desc = 'ROWS')) - 4);
	
	-- Defining the SQL statement for adding a new database file with the calculated size
	SET @SQLString = 'ALTER DATABASE ' + @dbName + ' ADD FILE (Name = ' + @dbName + '_ProtocolData, FILENAME = "'
	+ @dbPath + '_ProtocolData.ndf", SIZE = ' + CONVERT(nvarchar,@size)  + 'KB, MAXSIZE = UNLIMITED, FILEGROWTH = 1%) TO FILEGROUP Protocol;'	
	
	EXECUTE (@SQLString);
	RAISERROR('Secondary datafile was added.',0,1) WITH NOWAIT
END

IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = object_id(N'[dbo].[xProtocolData]') AND OBJECTPROPERTY(id, N'IsUserTable') = 1)
BEGIN	
	RAISERROR('Creating xProtocolData on secondary datafile...',0,1) WITH NOWAIT
	CREATE TABLE [dbo].[xProtocolData] (
		[FkIdent] [int] NOT NULL ,
		[FkMode] [int] NOT NULL ,
		[FkChannel] [int] NOT NULL ,
		[FkLocation] [int] NOT NULL ,
		[FkInstrument] [int] NOT NULL ,
		[FkParameter] [int] NOT NULL ,
		[Value] [float] NOT NULL ,
		[ULimit] [float] NOT NULL ,
		[LLimit] [float] NOT NULL ,
		[Position] [float] NOT NULL ,
		[FkUnit] [int] NOT NULL ,
		[FkExtra] [int] NOT NULL 
	) ON [Protocol]
	RAISERROR('xProtocolData was created on secondary datafile.',0,1) WITH NOWAIT			
END

GO
